<?php
namespace Ep\Hak\Service;

use Ep\Gath\Plugin\Products;
use Ep\Hak\HakService;
use Ep\Woo\Product\ProductInterface;
use Ep\WordPress\Service\AbstractWordPressService;
use WC_Coupon;
use WC_Meta_Data;
use WC_Order;
use WP_REST_Response;
use Zend\Stdlib\ArrayUtils;

/**
 * Class CouponService
 *
 * @package Ep\Hak
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class CouponService extends AbstractWordPressService
{
	public function attach(): void
	{
		add_filter('woocommerce_get_shop_coupon_data', [$this, 'getCouponData'], 10, 2);
		add_filter('woocommerce_coupon_is_valid', [$this, 'isCouponValid'], 10, 2);
		add_filter('woocommerce_coupon_message', [$this, 'getCouponMessage'], 10, 3);
		add_filter('woocommerce_cart_totals_coupon_label', [$this, 'getCouponLabel'], 10, 2);
		add_filter('woocommerce_cart_totals_coupon_html', [$this, 'getCouponHtml'], 10, 2);
		add_filter('woocommerce_rest_prepare_shop_order_object', [$this, 'prepareShopOrder'], 10, 2);
	}

	public function prepareShopOrder(WP_REST_Response $response, WC_Order $order)
    {
        if (
            in_array(HakService::COUPON_CODE, $order->get_used_coupons()) ||
            in_array(HakService::DYNAMIC_COUPON_CODE, $order->get_used_coupons())
        ) {
            $config = $this->getConfig('hak');

            foreach ($response->data['coupon_lines'] as $i => $couponLine) {
                if ($couponLine['code'] === HakService::COUPON_CODE) {
                    foreach ($couponLine['meta_data'] as $j => $couponMeta) {
                        $products = [];
                        foreach ($order->get_items() as $item) {
                            /** @noinspection PhpUndefinedMethodInspection */
                            $productId = intval($item->get_data()['product_id']);

                            if (!in_array($productId, $config['hakfilter']['activated_by'] ?? [])) {
                                $product = wc_get_product($productId);

                                if (is_object($product)) {
                                    $products[] = wc_get_product($productId)->get_sku();
                                }
                            }
                        }

                        /** @var WC_Meta_Data $couponMeta */
                        $data = $couponMeta->get_data();
                        if ('coupon_data' === $data['key']) {
                            $data['value']['name'] = $config['couponName'];
                            $data['value']['discount_type'] = 'percent_product';
                            $data['value']['products'] = $products;
                            $response->data['coupon_lines'][$i]['meta_data'][$j] = new WC_Meta_Data($data);
                            break;
                        }
                    }
                }
            }

            foreach ($response->data['coupon_lines'] as $i => $couponLine) {
                if ($couponLine['code'] === HakService::DYNAMIC_COUPON_CODE) {
                    foreach ($couponLine['meta_data'] as $j => $couponMeta) {
                        /** @var WC_Meta_Data $couponMeta */
                        $data = $couponMeta->get_data();
                        $products = [];
                        /** @var Products $gath */
                        $gath = apply_filters('ep_gath_products', null);

                        foreach ($order->get_items() as $item) {
                            /** @var \WC_Order_Item $item */
                            $product = wc_get_product($gath->getProductById($item->get_data()['product_id'])->ID);

                            if (in_array($product->get_id(), $config['hakfilter']['activated_by'] ?? [])) {
                                $products[] = $product->get_sku();
                            }
                        }

                        if ('coupon_data' === $data['key']) {
                            $data['value']['name'] = 'HAK Akcija';
                            $data['value']['discount_type'] = 'percent_product';
                            $data['value']['products'] = $products;
                            $response->data['coupon_lines'][$i]['meta_data'][$j] = new WC_Meta_Data($data);
                            break;
                        }
                    }
                }
            }
        }

        return $response;
    }

	/**
	 * @param $false
	 * @param $data
	 *
	 * @return mixed
	 */
	public function getCouponData($false, $data)
	{
		$config = $this->getConfig('hak');

		switch ($data) {
            case HakService::COUPON_CODE:
                $excludedProductIds = $config['hakfilter']['activated_by'] ?? [];

                foreach ($config['exclude_brands'] ?? [] as $brandId) {
                    $excludedProductIds = ArrayUtils::merge($excludedProductIds, $this->getExcludedProductIdsByBrand($brandId));
                }

                $cart = \WC()->cart;

                foreach ($cart->get_cart_contents() as $item) {
                    /** @var \WC_Product $product */
                    $product = $item['data'];

                    if ($product->get_id() > Products::ID_OFFSET_TIRES) {
                        $excludedProductIds[] = $product->get_id();
                    }
                }

/*
                $excludedProductIds = array_merge($excludedProductIds, array_map(function($p) {
                    return $p['id'];
                }, get_option('tecdoc_filter_')));
*/
                return [
                    'name' => HakService::COUPON_CODE,
                    'discount_type' => 'percent_product',
                    'amount' => intval($config['discountPercentage']),
                    'excluded_product_ids' => $excludedProductIds,
                ];
            case HakService::DYNAMIC_COUPON_CODE:
                $excludedProductIds = $config['hakfilter']['excluded_product_ids'] ?? [];

                foreach ($config['exclude_brands'] ?? [] as $brandId) {
                    $excludedProductIds = ArrayUtils::merge($excludedProductIds, $this->getExcludedProductIdsByBrand($brandId));
                }

                return [
                    'name' => HakService::DYNAMIC_COUPON_CODE,
                    'discount_type' => 'percent_product',
                    'amount' => $this->calculateDynamicCouponAmount(),
                    'excluded_product_ids' => $config['hakfilter']['excluded_product_ids'] ?? [],
                    'product_ids' => $config['hakfilter']['activated_by'] ?? [],
                ];
        }

		return $false;
	}

	private function getExcludedProductIdsByBrand(int $brandId)
    {
        $cart = \WC()->cart;
        $excludedProductIds = [];

        foreach ($cart->get_cart_contents() as $item) {
            /** @var ProductInterface $product */
            $product = $item['data'];

            if (($brand = $product->getBrand()) && $brand['brandid'] === $brandId) {
                $excludedProductIds[] = $product->getId();
            }
        }

        return $excludedProductIds;
    }

	private function calculateDynamicCouponAmount(): float
    {
        $items = $this->getCartItemsEligibleForDynamicCoupon();

        foreach ($items as $item) {
            /** @var ProductInterface $product */
            $product = $item['data'];

            $pop = 1 - ($product->getDiscountPercentage() / 100);
            return (1 - (0.6 / $pop)) * 100;
        }

        return 0;
    }

    /**
     * Vraća sve proizvode i njihove količine iz košarice koji su validni za dinamički
     * popust.
     *
     * @return array
     */
    private function getCartItemsEligibleForDynamicCoupon()
    {
        $cart = \WC()->cart;
        $config = $this->getConfig('hak');
        $items = [];

        foreach ($cart->get_cart_contents() as $item) {
            /** @var \WC_Product $product */
            $product = $item['data'];

            if (
                in_array($product->get_id(), $config['hakfilter']['activated_by'] ?? []) &&
                !in_array($product->get_id(), $config['hakfilter']['excluded_product_ids'] ?? [])
            ) {
                $items[] = $item;
            }
        }

        return $items;
    }

	/**
	 * @param $label
	 * @param WC_Coupon $coupon
	 *
	 * @return string
	 */
	public function getCouponLabel($label, WC_Coupon $coupon)
	{
		$config = $this->getConfig('hak');

		switch ($coupon->get_code('')) {
            case HakService::COUPON_CODE:
                return esc_html($config['couponName']);
            case HakService::DYNAMIC_COUPON_CODE:
                return 'Dodatni akcijski popust za HAK članove';
        }

		return $label;
	}

	/**
	 * @param $html
	 * @param WC_Coupon $coupon
	 *
	 * @return mixed
	 */
	public function getCouponHtml($html, WC_Coupon $coupon)
	{
		if (in_array($coupon->get_code(''), [HakService::COUPON_CODE, HakService::DYNAMIC_COUPON_CODE])) {
			return preg_replace('#<a[^<]+</a>$#', '', $html);
		}

		return $html;
	}

	/**
	 * @param $message
	 * @param $messageCode
	 * @param WC_Coupon $coupon
	 *
	 * @return string
	 */
	public function getCouponMessage($message, $messageCode, WC_Coupon $coupon)
	{
		if (in_array($coupon->get_code(''), [HakService::COUPON_CODE, HakService::DYNAMIC_COUPON_CODE])) {
			switch ($messageCode) {
				case WC_Coupon::WC_COUPON_SUCCESS:
					return 'HAK popust uspješno primjenjen.';
				case WC_Coupon::WC_COUPON_REMOVED:
					return 'HAK popust uklonjen.';
			}
		}

		return $message;
	}

	/**
	 * Za HAK kupon provjerava da li je korisnik član HAK-a.
	 *
	 * @param $valid
	 * @param WC_Coupon $coupon
	 *
	 * @return bool
	 */
	public function isCouponValid($valid, WC_Coupon $coupon)
    {
        /** @var HakService $hak */
        $hak = $this->services->get('Hak');

        switch ($coupon->get_code('')) {
            case HakService::COUPON_CODE:
                return $hak->validate();
            case HakService::DYNAMIC_COUPON_CODE:
                $items = $this->getCartItemsEligibleForDynamicCoupon();
                return $hak->validate() && count($items) > 0;
        }

        return $valid;
    }
}
